package com.fjhx.account.service.cost.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.account.entity.cost.dto.CostControlDto;
import com.fjhx.account.entity.cost.dto.CostControlSelectDto;
import com.fjhx.account.entity.cost.po.CostControl;
import com.fjhx.account.entity.cost.po.CostControlDetail;
import com.fjhx.account.entity.cost.vo.CostControlDetailVo;
import com.fjhx.account.entity.cost.vo.CostControlVo;
import com.fjhx.account.mapper.cost.CostControlMapper;
import com.fjhx.account.service.cost.CostControlDetailService;
import com.fjhx.account.service.cost.CostControlService;
import com.fjhx.common.controller.Label;
import com.fjhx.common.service.logistics.LogisticsCompanyInfoService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;


/**
 * <p>
 * 费控 服务实现类
 * </p>
 *
 * @author
 * @since 2024-03-18
 */
@Service
public class CostControlServiceImpl extends ServiceImpl<CostControlMapper, CostControl> implements CostControlService {
    @Autowired
    private LogisticsCompanyInfoService logisticsCompanyInfoService;
    @Autowired
    private CostControlDetailService costControlDetailService;

    @Override
    public Page<CostControlVo> getPage(CostControlSelectDto dto) {
        IWrapper<CostControl> wrapper = getWrapper();
        wrapper.orderByDesc("cc", CostControl::getId);
        Page<CostControlVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public CostControlVo detail(Long id) {
        CostControl CostControl = this.getById(id);
        CostControlVo result = BeanUtil.toBean(CostControl, CostControlVo.class);

        //赋值详情
        setInfo(Arrays.asList(result));

        //赋值明细列表
        List<CostControlDetailVo> costControlDetailVos = costControlDetailService.getList(IWrapper.getWrapper().eq(CostControlDetail::getCostControlId, result.getId()));
        result.setCostControlDetailList(costControlDetailVos);

        return result;
    }

    private void setInfo(List<CostControlVo> costControlVos) {
        //赋值物流公司信息
        logisticsCompanyInfoService.attributeAssign(costControlVos, CostControlVo::getLogisticsCompanyId, (detail, logistics) -> {
            detail.setLogisticsName(logistics.getName());
            detail.setLogisticsTaxPoints(logistics.getTaxPoints());
            detail.setLogisticsAccountName(logistics.getAccountName());
            detail.setLogisticsAccountBank(logistics.getAccountBank());
            detail.setLogisticsAccountNumber(logistics.getAccountNumber());
        });
        //赋值申请人信息
        UserUtil.assignmentNickName(costControlVos, CostControlVo::getApplyUserId, CostControlVo::setApplyUserName);
    }

    @Override
    public void add(CostControlDto costControlDto) {
        this.save(costControlDto);
    }

    @Override
    public void edit(CostControlDto costControlDto) {
        this.updateById(costControlDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public JSONObject getFieldInfo() {


        List<JSONObject> infoField = getField(CostControlVo.class);
        List<JSONObject> detailField = getField(CostControlDetailVo.class);

        JSONObject json = new JSONObject();
        json.put("infoField", infoField);
        json.put("detailField", detailField);


        return json;
    }

    private List<JSONObject> getField(Class cl) {
        //获取所有字段
        List<Field> fieldList = new ArrayList<>();
        Collections.addAll(fieldList, cl.getDeclaredFields());
        Class<? super CostControlVo> superclass = cl.getSuperclass();
        if (ObjectUtil.isNotEmpty(superclass)) {
            Collections.addAll(fieldList, superclass.getDeclaredFields());
        }

        //遍历处理
        List<JSONObject> arr = new ArrayList<>();
        for (Field field : fieldList) {
            Label labelAnnotation = field.getAnnotation(Label.class);
            if (labelAnnotation != null) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("key", field.getName());
                jsonObject.put("val", labelAnnotation.value());
                arr.add(jsonObject);
            }
        }
        return arr;
    }

}
